# IEDMod release 2 by David Bowland
# ./addons/eventscripts/iedmod/iedmod.py

# >>> To configure this addon please see iedmod.cfg <<<

"""
Allows players to create props which explode when players are near. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'IEDMod'
info.version = '2'
info.url = 'http://addons.eventscripts.com/addons/view/iedmod'
info.basename = 'iedmod'
info.author = 'SuperDave'


str_last_sound = ''
list_ied_models = []
list_file_downloads = []
dict_say_commands = {'purchase':'', 'create':''}
dict_players = {}
dict_ieds = {}

tuple_ied_sounds = ('buttons/blip1.wav', 'buttons/button14.wav', 'buttons/button15.wav', 'buttons/button16.wav', 'buttons/button17.wav', 'buttons/button18.wav', 'buttons/button24.wav', 'buttons/button3.wav', 'buttons/button9.wav', 'npc/roller/remote_yes.wav', 'npc/turret_floor/ping.wav')
dict_events = {'ied_activated':{'userid':'short', 'index':'short'}, 'ied_exploded':{'userid':'short', 'attacker':'short', 'index':'short'}, 'ied_planted':{'userid':'short', 'index':'short'}, 'ied_purchased':{'userid':'short', 'count':'short'}}
dict_options = {'iedmod_detonation_radius':[150, 'IEDs will be detonated when a player is within this distance'], 'iedmod_detonation_checkdelay':[1, 'Number of seconds between checking the distance of players'], 'iedmod_detonation_team':[0, '0 = Only opposing team members can detonate an IED, 1 = Any player can detonate an IED'], 'iedmod_explosion_radius':[350, 'Radius of IED explosion--HE grenade is 350'], 'iedmod_explosion_magnitude':[200, 'Magnitude of IED explosion--HE grenade is 100'], 'iedmod_explosion_sound':[40, 'Percent of the map that can hear an IED explode'], 'iedmod_purchase_cost':[2000, 'Cost of purchasing an IED--0 will give all players the maximum number of IEDs'], 'iedmod_purchase_max':[3, 'Maximum number of IEDs a player can have--0 for no limit'], 'iedmod_purchase_command':['!buyied', 'Command players must say to purchase an IED'], 'iedmod_purchase_buyzone':[1, '0 = IEDs can be puchased anywhere, 1 = IEDs can only be puchased in buy zones'], 'iedmod_purchase_loseafterdeath':[1, '0 = players keep IEDs that have not been planted when killed, 1 = players will lose IEDs that have not been planted when killed'], 'iedmod_create_command':['!ied', 'Command players must say to plant an IED'], 'iedmod_create_delay':[5, 'Number of seconds a player must wait after creating an IED to create another--set to 0 for no delay'], 'iedmod_create_knife':[1, '0 = players can create an IED with any active weapon, 1 = players are required to have the knife as active weapon to create an IED, 2 = players will be forced to have the knife as active weapon when creating and IED [default 1]'], 'iedmod_create_sound':[40, 'Percent of the map that can hear an IED beep when activated--set to 0 for no sound'], 'iedmod_bot_allowied':[2, 'Number of IEDs bot will be allowed to buy or plant per round'], 'iedmod_bot_purchasechance':[80, 'Percent chance a bot will purchase an IED if it has enough money'], 'iedmod_bot_createdelay_min':[5, 'Minumim number of seconds after spawning a bot will wait to create an IED'], 'iedmod_bot_createdelay_max':[45, 'Maximum number of seconds after spawning a bot will wait to create an IED'], 'iedmod_bot_iterationstoremove':[4, 'Number of IED distance updates until an IED spawned by a bot is removed if that bot is stuck in the IED'], 'iedmod_announce_spawn':[1, '0 = no change, 1 = announce to each player his or her IED count as well as how to purchase and create IEDs when the player spawns'], 'iedmod_announce_purchase_chat':[0, '0 = no change, 1 = announce IED purchasing feedback in chat area'], 'iedmod_announce_purchase_center':[1, '0 = no change, 1 = announce IED purchasing feedback in center text'], 'iedmod_announce_purchase_hud':[1, '0 = no change, 1 = announce IED purchasing feedback in a HUD message'], 'iedmod_announce_create_chat':[0, '0 = no change, 1 = announce IED creation feedback in chat area'], 'iedmod_announce_create_center':[1, '0 = no change, 1 = announce IED creation feedback in center text'], 'iedmod_announce_create_hud':[1, '0 = no change, 1 = announce IED creation feedback in a HUD message']}
func_lang_text = None


def load():
   """
   Creates commands for settings
   Checks to make sure critical server variables are created by the config
   Creates say commands for purchasing and creating IEDs
   Calls es_map_start
   Calls player_spawn for each live player
   """
   global dict_say_commands
   global func_lang_text

   if not es.exists('command', 'iedmod_add'):
      es.regcmd('iedmod_add', 'iedmod/model_cmd', 'iedmod_add <model relative to ./models/> [0 = force client download of model, 1 = clients do not need to download model]\nServer command for adding IED models')
   if not es.exists('command', 'iedmod_remove'):
      es.regcmd('iedmod_remove', 'iedmod/model_cmd', 'iedmod_remove <model relative to ./models/>\nServer command for removing IED models')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('iedmod') + '/iedmod.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/iedmod/iedmod.cfg')
   else:
      es.dbgmsg(0, 'IEDMod: Unable to load iedmod.cfg! Please ensure it is in the ./iedmod/ directory.')

   str_lang_path = es.getAddonPath('iedmod') + '/iedmod_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'IEDMod: Unable to load iedmod_languages.ini! Please ensure it is in the ./iedmod/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   str_command = str(es.ServerVar('iedmod_purchase_command'))
   if str_command:
      if not es.exists('saycommand', str_command):
         es.regsaycmd(str_command, 'iedmod/buy_ied_cmd', 'Say command for purchasing an IED')
         dict_say_commands['purchase'] = str_command
   str_command = str(es.ServerVar('iedmod_create_command'))
   if str_command:
      if not es.exists('saycommand', str_command):
         es.regsaycmd(str_command, 'iedmod/create_ied_cmd', 'Say command for creating an IED')
         dict_say_commands['create'] = str_command

   es_map_start({})

   for int_userid in es.getUseridList():
      if not es.getplayerprop(int_userid, 'CBasePlayer.pl.deadflag'):
         player_spawn({'userid':str(int_userid), 'es_steamid':es.getplayersteamid(int_userid), 'es_userteam':str(es.getplayerteam(int_userid))})


def es_map_start(event_var):
   """
   Cancels outstanding delays from the previous map
   Clears the dictionary of player
   Calls load_events
   Adds files that need to be downloaded by clients to the "downloadables" string table
   """
   global dict_players

   for int_userid in dict_players:
      if dict_players[int_userid]['prevent_create']:
         gamethread.cancelDelayed('iedmod_player_%s' % int_userid)

   dict_players.clear()

   load_events()

   for str_file in list_file_downloads:
      es.stringtable('downloadables', str_file)


def round_start(event_var):
   """
   Removes outstanding IEDs and corresponding delays
   Removes outstanding IED creation delays
   Clears the dictionary of IEDs
   """
   global dict_players
   global dict_ieds

   dict_active_entites = es.createentitylist()
   for int_userid in dict_players:
      dict_current_player = dict_players[int_userid]
      list_active_ieds = dict_current_player['active_ieds']
      for int_ied_index in list_active_ieds:
         if dict_active_entites.has_key(int_ied_index):
            es.server.cmd('es_xremove %s' % int_ied_index)
      list_active_ieds[:] = []

      if dict_current_player['prevent_create'] == 1:
         gamethread.cancelDelayed('iedmod_player_%s' % int_userid)
         dict_current_player['prevent_create'] = 0

   if dict_ieds:
      gamethread.cancelDelayed('iedmod_updateloop')
   dict_ieds.clear()


def player_spawn(event_var):
   """
   Creates the player in the player dictionary
   Calls spawn_announce
   Handles purchase and creation of IEDs by bots
   """
   global dict_players

   int_userid = int(event_var['userid'])
   if not dict_players.has_key(int_userid):
      dict_players[int_userid] = {'ied_count':0, 'active_ieds':[], 'prevent_create':0}

   dict_current_player = dict_players[int_userid]
   if int(event_var['es_userteam']) > 1:
      int_ied_cost = int(es.ServerVar('iedmod_purchase_cost'))
      if not int_ied_cost:
         dict_current_player['ied_count'] = int(es.ServerVar('iedmod_purchase_max'))

      spawn_announce(int_userid)

      int_allow_ied = int(es.ServerVar('iedmod_bot_allowied'))
      if event_var['es_steamid'] == 'BOT' and int_allow_ied:
         if dict_current_player['prevent_create']:
            gamethread.cancelDelayed('iedmod_player_%s' % int_userid)
         if int_ied_cost:
            int_money = es.getplayerprop(int_userid, 'CCSPlayer.m_iAccount')
            int_purchase_chance = int(es.ServerVar('iedmod_bot_purchasechance'))
            int_eco_limit = int(es.ServerVar('bot_eco_limit'))
            for int_x in range(dict_current_player['ied_count'], int_allow_ied):
               if random.randint(1, 100) <= int_purchase_chance and int_money >= int_ied_cost + int_eco_limit:
                  purchase_ied(int_userid, int_money, int_ied_cost)
                  int_money -= int_ied_cost
               else:
                  break
         else:
            dict_current_player['ied_count'] = int_allow_ied
         if dict_current_player['ied_count']:
            dict_current_player['prevent_create'] = -1
            gamethread.delayedname(random.randint(es.ServerVar('iedmod_bot_createdelay_min'), es.ServerVar('iedmod_bot_createdelay_max')), 'iedmod_player_%s' % int_userid, bot_create_ied, int_userid)


def player_death(event_var):
   """
   Removes the dead player's delay
   Removes the dead player's IED count
   """
   global dict_players

   int_userid = int(event_var['userid'])
   if dict_players.has_key(int_userid):
      dict_current_player = dict_players[int_userid]
      if dict_current_player['prevent_create']:
         gamethread.cancelDelayed('iedmod_player_%s' % int_userid)
         dict_current_player['prevent_create'] = 0

      if int(es.ServerVar('iedmod_purchase_loseafterdeath')):
         dict_current_player['ied_count'] = 0


def player_disconnect(event_var):
   """Removes the disconneting player's IEDs and delays"""
   global dict_players
   global dict_ieds

   int_userid = int(event_var['userid'])
   if dict_players.has_key(int_userid):
      list_active_ieds = dict_players[int_userid]['active_ieds']
      for int_ied_index in list_active_ieds:
         gamethread.cancelDelayed('iedmod_ied_%s' % int_ied_index)
         es.server.cmd('es_xremove %s' % int_ied_index)
         del dict_ieds[int_ied_index]
      if dict_players[int_userid]['prevent_create']:
         gamethread.cancelDelayed('iedmod_player_%s' % int_userid)
      del dict_players[int_userid]


def unload():
   """
   Removes active IEDs and delays
   Cancels ied update delay
   Unregisters say commands
   """
   dict_active_entites = es.createentitylist()
   for int_userid in dict_players:
      dict_current_player = dict_players[int_userid]
      list_active_ieds = dict_current_player['active_ieds']
      for int_ied_index in list_active_ieds:
         if dict_active_entites.has_key(int_ied_index):
            es.server.cmd('es_xremove %s' % int_ied_index)
      if dict_current_player['prevent_create']:
         gamethread.cancelDelayed('iedmod_player_%s' % int_userid)

   if dict_ieds:
      gamethread.cancelDelayed('iedmod_updateloop')

   for str_command_type in dict_say_commands:
      str_command = dict_say_commands[str_command_type]
      if str_command:
         es.unregsaycmd(str_command)

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def buy_ied_cmd():
   """
   Calls purchase_ied
   """
   int_userid = es.getcmduserid()
   if es.getplayerteam(int_userid) > 1 and not es.getplayerprop(int_userid, 'CBasePlayer.pl.deadflag'):
      if es.getplayerprop(int_userid, 'CCSPlayer.m_bInBuyZone') % 2 or es.getentityindex('func_buyzone') == -1 or not int(es.ServerVar('iedmod_purchase_buyzone')):
         int_ied_max = int(es.ServerVar('iedmod_purchase_max'))
         if dict_players[int_userid]['ied_count'] < int_ied_max or not int_ied_max:
            int_money = es.getplayerprop(int_userid, 'CCSPlayer.m_iAccount')
            int_cost = int(es.ServerVar('iedmod_purchase_cost'))
            if int_cost <= int_money and int_cost:
               purchase_ied(int_userid, int_money, int_cost)

            else:
               announce_type('purchase', int_userid, 'purchase cash', {'cost':int_cost})
         else:
            announce_type('purchase', int_userid, 'purchase max', {})
      else:
         announce_type('purchase', int_userid, 'purchase buy zone', {})
   else:
      announce_type('purchase', int_userid, 'purchase dead', {})


def create_ied_cmd():
   """
   Checks the player for an IED to create
   Calls create_ied
   """
   int_userid = es.getcmduserid()
   if es.getplayerteam(int_userid) > 1 and not es.getplayerprop(int_userid, 'CBasePlayer.pl.deadflag'):
      dict_current_player = dict_players[int_userid]
      if not dict_current_player['prevent_create']:
         int_ied_max = int(es.ServerVar('iedmod_purchase_max'))
         if dict_current_player['ied_count'] or not int_ied_max:
            int_require_knife = int(es.ServerVar('iedmod_create_knife'))
            if int_require_knife == 2:
               es.sexec(int_userid, 'use weapon_knife')
               int_require_knife = 0
            if es.createplayerlist(int_userid)[int_userid]['weapon'] == 'weapon_knife' or not int_require_knife:
               create_ied(int_userid, True)

            else:
               announce_type('create', int_userid, 'create knife', {})
         else:
            announce_type('create', int_userid, 'create empty', {})
      else:
         announce_type('create', int_userid, 'create limit', {})
   else:
      announce_type('create', int_userid, 'create dead', {})


def purchase_ied(int_userid, int_money, int_cost):
   """
   Increases the player's IED count
   Displays IED purchase status
   """
   global dict_players

   dict_current_player = dict_players[int_userid]
   es.setplayerprop(int_userid, 'CCSPlayer.m_iAccount', int_money - int_cost)
   dict_current_player['ied_count'] += 1

   es.event('initialize', 'ied_purchased')
   es.event('setint', 'ied_purchased', 'userid', int_userid)
   es.event('setint', 'ied_purchased', 'count', dict_current_player['ied_count'])
   es.event('fire', 'ied_purchased')

   if announce_type('purchase', int_userid, 'purchase success', {}):
      spawn_announce(int_userid)


def create_ied(int_userid, bool_is_human):
   """
   Creates the IED and starts the IED loop
   Starts the delay to prevent players from creating a second IED too quickly
   Announces IED creation status
   """
   global dict_players
   global dict_ieds

   dict_current_player = dict_players[int_userid]
   if dict_current_player['ied_count']:
      dict_current_player['ied_count'] -= 1

   es.server.cmd('es_xprop_physics_create %s %s' % (int_userid, random.choice(list_ied_models)))
   int_ied_index = int(es.ServerVar('eventscripts_lastgive'))
   dict_current_player['active_ieds'].append(int_ied_index)
   if not dict_ieds:
      gamethread.delayedname(es.ServerVar('iedmod_detonation_checkdelay'), 'iedmod_updateloop', ied_loop)
   dict_ieds[int_ied_index] = {'owner':int_userid, 'opposing_team':5 - es.getplayerteam(int_userid), 'prevent_explosion':1}

   int_creation_delay = int(es.ServerVar('iedmod_create_delay'))
   if int_creation_delay and bool_is_human:
      dict_current_player['prevent_create'] = 1
      gamethread.delayedname(int_creation_delay, 'iedmod_player_%s' % int_userid, remove_restriction, int_userid)

   es.event('initialize', 'ied_planted')
   es.event('setint', 'ied_planted', 'userid', int_userid)
   es.event('setint', 'ied_planted', 'index', int_ied_index)
   es.event('fire', 'ied_planted')

   announce_type('create', int_userid, 'create success', {})
   if int(es.ServerVar('iedmod_announce_spawn')):
      int_max = int(es.ServerVar('iedmod_purchase_max'))
      if int_max:
         es.tell(int_userid, '#multi', func_lang_text('ied count', {'count':dict_current_player['ied_count'], 'max':int_max}, playerlib.getPlayer(int_userid).get('lang')))
      else:
         es.tell(int_userid, '#multi', func_lang_text('ied count no max', {'count':dict_current_player['ied_count']}, playerlib.getPlayer(int_userid).get('lang')))


def ied_loop():
   """
   Activates or removes IED
   Checks the IED for players inside the detonation radius
   Detonates the IED and removes the prop
   """
   global str_last_sound
   global dict_players
   global dict_ieds

   list_ieds_to_remove = []
   dict_prop_physics = es.createentitylist('prop_physics')
   dict_player_list = es.createplayerlist()
   for int_ied_index in dict_ieds:
      dict_current_ied = dict_ieds[int_ied_index]
      int_ied_owner = dict_current_ied['owner']
      int_opposing_team = dict_current_ied['opposing_team']
      dict_current_player = dict_players[int_ied_owner]
      if dict_prop_physics.has_key(int_ied_index):
         str_ied_origin = es.getindexprop(int_ied_index, 'CBaseEntity.m_vecOrigin')
         list_ied_location = str_ied_origin.split(',')
         if dict_current_ied['prevent_explosion']:
            tuple_player_location = es.getplayerlocation(int_ied_owner)
            if pow(pow(float(list_ied_location[0]) - tuple_player_location[0], 2) + pow(float(list_ied_location[1]) - tuple_player_location[1], 2) + pow(float(list_ied_location[2]) - tuple_player_location[2], 2), 0.5) < es.ServerVar('iedmod_detonation_radius'):
               dict_current_ied['prevent_explosion'] += 1
               if es.getplayersteamid(int_ied_owner) == 'BOT' and dict_current_ied['prevent_explosion'] >= es.ServerVar('iedmod_bot_iterationstoremove'):
                  dict_current_player['active_ieds'].remove(int_ied_index)
                  list_ieds_to_remove.append(int_ied_index)
                  es.server.cmd('es_xremove %s' % int_ied_index)
               continue

            else:
               dict_current_ied['prevent_explosion'] = 0

               es.event('initialize', 'ied_activated')
               es.event('setint', 'ied_activated', 'userid', int_ied_owner)
               es.event('setint', 'ied_activated', 'index', int_ied_index)
               es.event('fire', 'ied_activated')

               float_beep_sound = float(es.ServerVar('iedmod_create_sound'))
               if float_beep_sound:
                  list_possible_sounds = list(tuple_ied_sounds)
                  if str_last_sound in list_possible_sounds:
                     list_possible_sounds.remove(str_last_sound)
                  str_last_sound = random.choice(list_possible_sounds)
                  es.emitsound('entity', int_ied_index, str_last_sound, 1.0, (100 - float_beep_sound) / 100)

         for int_userid in dict_player_list:
            dict_loop_player = dict_player_list[int_userid]
            int_teamid = dict_loop_player['teamid']
            if (int_teamid == int_opposing_team or (int(es.ServerVar('iedmod_detonation_team')) and int_teamid in (2, 3))) and not dict_loop_player['isdead']:
               if pow(pow(float(list_ied_location[0]) - dict_loop_player['x'], 2) + pow(float(list_ied_location[1]) - dict_loop_player['y'], 2) + pow(float(list_ied_location[2]) - dict_loop_player['z'], 2), 0.5) < es.ServerVar('iedmod_detonation_radius'):
                  for int_explosion_index in es.createentitylist('env_explosion'):
                     es.server.cmd('es_xremove %s' % int_explosion_index)
                  float_explode_sound = float(es.ServerVar('iedmod_explosion_sound'))
                  if float_explode_sound:
                     es.emitsound('entity', int_ied_index, 'weapons/hegrenade/explode%s.wav' % random.randint(3, 5), '1.0', (100 - float_explode_sound) / 100)
                  dict_current_player['active_ieds'].remove(int_ied_index)
                  list_ieds_to_remove.append(int_ied_index)
                  es.server.cmd('es_xremove %s' % int_ied_index)

                  es.server.cmd('es_xgive %s env_explosion' % int_userid)
                  int_explosion_index = int(es.ServerVar('eventscripts_lastgive'))
                  es.server.cmd('es_xfire %s env_explosion addoutput \"imagnitude %s\"' % (int_userid, es.ServerVar('iedmod_explosion_magnitude')))
                  es.server.cmd('es_xfire %s env_explosion addoutput \"iradiusoverride %s\"' % (int_userid, es.ServerVar('iedmod_explosion_radius')))
                  es.setindexprop(int_explosion_index, 'CBaseEntity.m_vecOrigin', str_ied_origin)
                  es.setindexprop(int_explosion_index, 'CBaseEntity.m_hOwnerEntity', es.getplayerhandle(int_ied_owner))
                  es.server.cmd('es_xfire %s env_explosion explode' % int_userid)
                  es.server.cmd('es_xfire %s env_explosion kill' % int_userid)

                  es.event('initialize', 'ied_exploded')
                  es.event('setint', 'ied_exploded', 'userid', int_userid)
                  es.event('setint', 'ied_exploded', 'attacker', int_ied_owner)
                  es.event('setint', 'ied_exploded', 'index', int_ied_index)
                  es.event('fire', 'ied_exploded')
                  break

      else:
         if int_ied_index in dict_current_player['active_ieds']:
            dict_current_player['active_ieds'].remove(int_ied_index)
         list_ieds_to_remove.append(int_ied_index)

   for int_ied_index in list_ieds_to_remove:
      del dict_ieds[int_ied_index]

   if dict_ieds:
      gamethread.delayedname(es.ServerVar('iedmod_detonation_checkdelay'), 'iedmod_updateloop', ied_loop)


def remove_restriction(int_userid):
   """Removes the restriction for creating another IED"""
   global dict_players

   if dict_players.has_key(int_userid):
      dict_players[int_userid]['prevent_create'] = 0


def bot_create_ied(int_userid):
   """Forces the provided bot to create an IED"""
   global dict_players

   if dict_players.has_key(int_userid):
      dict_current_player = dict_players[int_userid]
      dict_current_player['prevent_create'] = 0

      if int(es.ServerVar('iedmod_create_knife')):
         es.sexec(int_userid, 'use weapon_knife')
      create_ied(int_userid, False)

      if dict_current_player['ied_count']:
         dict_current_player['prevent_create'] = -1
         gamethread.delayedname(random.randint(es.ServerVar('iedmod_bot_createdelay_min'), es.ServerVar('iedmod_bot_createdelay_max')), 'iedmod_player_%s' % int_userid, bot_create_ied, int_userid)


def announce_type(str_announce_type, int_userid, str_text, dict_tokens):
   """
   Announces the provided text in the desired manner
   Returns: False = no text was displayed, True = at least one announce method was used to display text
   """
   bool_displayed = False
   str_player_lang = playerlib.getPlayer(int_userid).get('lang')
   if int(es.ServerVar('iedmod_announce_%s_chat' % str_announce_type)):
      es.tell(int_userid, '#multi', func_lang_text(str_text, dict_tokens, str_player_lang))
      bool_displayed = True
   if int(es.ServerVar('iedmod_announce_%s_center' % str_announce_type)):
      es.centertell(int_userid, remove_tags(func_lang_text(str_text, dict_tokens, str_player_lang)))
      bool_displayed = True
   if int(es.ServerVar('iedmod_announce_%s_hud' % str_announce_type)):
      hud_hint(int_userid, remove_tags(func_lang_text(str_text, dict_tokens, str_player_lang)))
      bool_displayed = True
   return bool_displayed


def spawn_announce(int_userid):
   """
   Displays the player's IED count
   Displays instructions on purchasing and creating IEDs
   """
   if int(es.ServerVar('iedmod_announce_spawn')):
      dict_current_player = dict_players[int_userid]
      str_player_lang = playerlib.getPlayer(int_userid).get('lang')
      int_max = int(es.ServerVar('iedmod_purchase_max'))
      if int_max:
         es.tell(int_userid, '#multi', func_lang_text('ied count', {'count':dict_current_player['ied_count'], 'max':int_max}, str_player_lang))
      else:
         es.tell(int_userid, '#multi', func_lang_text('ied count no max', {'count':dict_current_player['ied_count']}, str_player_lang))

      dict_tokens = dict_say_commands.copy()
      dict_tokens['cost'] = int(es.ServerVar('iedmod_purchase_cost'))
      if (dict_current_player['ied_count'] < int_max or not int_max) and dict_tokens['cost']:
         if dict_current_player['ied_count'] or not int_max:
            es.tell(int_userid, '#multi', func_lang_text('ied create or purchase', dict_tokens, str_player_lang))
         else:
            es.tell(int_userid, '#multi', func_lang_text('ied purchase', dict_tokens, str_player_lang))
      else:
         es.tell(int_userid, '#multi', func_lang_text('ied create', dict_tokens, str_player_lang))


def model_cmd():
   """
   iedmod_add <model relative to ./models/> [0 = force client download of model, 1 = clients do not need to download model]
   iedmod_remove <model relative to ./models/>
   Adds or removes models from the list of IED models
   """
   global list_ied_models
   global list_file_downloads

   str_command = es.getargv(0).lower()
   int_arg_count = es.getargc()
   if int_arg_count in {'iedmod_add':(2, 3), 'iedmod_remove':(2, )}[str_command]:
      str_file = es.getargv(1)
      if not str_file.lower().endswith('.mdl'):
         str_file += '.mdl'
      if str_command == 'iedmod_add':
         if str_file not in list_ied_models:
            list_ied_models.append(str_file)
         if int_arg_count == 3:
            if not int(es.getargv(2)):
               list_file_downloads.append(str_file)

      else:
         if str_file in list_ied_models:
            list_ied_models.remove(str_file)
         if str_file in list_file_downloads:
            list_file_downloads.remove(str_file)

   else:
      es.dbgmsg(0, 'Syntax: ' + {'iedmod_add':'iedmod_add <model relative to ./models/> [0 = force client download of model, 1 = clients do not need to download model]', 'iedmod_remove':'iedmod_remove <model relative to ./models/>'}[str_command])


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('iedmod') + '/iedmod.res'):
      es.loadevents('declare', 'addons/eventscripts/iedmod/iedmod.res')
   else:
      if not os.path.isfile(es.getAddonPath('iedmod') + 'es_iedmod_events_db.txt'):
         es.keygroupcreate('iedmod_events')
         for str_event in dict_events:
            es.keycreate('iedmod_events', str_event)
            for str_event_var in dict_events[str_event]:
               es.keysetvalue('iedmod_events', str_event, str_event_var, dict_events[str_event][str_event_var])
         es.keygroupsave('iedmod_events', '|iedmod')
         es.keygroupdelete('iedmod_events')
      es.loadevents('declare', 'addons/eventscripts/iedmod/es_iedmod_events_db.txt')
      es.dbgmsg(0, 'IEDMod: Unable to load iedmod.res! Please ensure it is in the ./iedmod/ directory.')


def hud_hint(int_userid, str_message):
   """Sends the player a HUD hint"""
   es.usermsg('create', 'iedmod_hud', 'HintText')
   es.usermsg('write', 'short', 'iedmod_hud', -1)
   es.usermsg('write', 'string', 'iedmod_hud', str_message)
   es.usermsg('send', 'iedmod_hud', int_userid)
   es.usermsg('delete', 'iedmod_hud')


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'IEDMod: iedmod_languages.ini not found'